<?php
/**
 * Copyright (C) 2010 Convio, Inc. All rights reserved.
 *
 * This program is free open source software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or (at your 
 * option) any later version. The GNU General Public License is available at 
 * http://www.gnu.org/licenses/. 
 *
 * =====================================================
 *
 * @file Single-sign-on (SSO), Constituent personalization implementation.
 * @version $Id: ConvioUser.inc,v 1.1 2010/01/26 00:13:03 mpih Exp $
 */

class ConvioLoginHandler {

  private static $instance = NULL;  

  /**
   * Fetches the singleton ConvioLoginHandler instance.
   */
  public static function getInstance() {
    if (self::$instance === NULL) {
      self::$instance = new ConvioLoginHandler();
    }
    return self::$instance;
  }

  function ConvioLoginHandler() {}

  /**
   * Gets the user tied to the current HTTP session.
   *
   * @return the authenticated user, or NULL for the anonymous user
   */
  public function getUser() {
    $cons_id = $_SESSION[ConvioConstants::$SESSION_CONS_ID];
    if ($cons_id) {
      // We think the user is authenticated.

      // Cache the result for the duration of the request.
      global $convio_user;
      if (! isset($convio_user)) {
        $convio_user = ConvioUser::find($cons_id);
      }

      return $convio_user;

    } else {
      // Anonymous user.

      return NULL;
    }
  }

  /**
   * Logs out the current user and redirects to the home page.
   */
  public function logout($local_only = FALSE) {
    global $user;

    // Load the anonymous user
    $user = drupal_anonymous_user();

    if ($local_only === TRUE) {
      // Local logout only.

      drupal_goto();
      return;

    } else {
      // Additionally log out of COM.

      // TODO: asynchronous redirect to COM for a more fault-tolerant SSO experience.

      $_SESSION[ConvioConstants::$SESSION_CONS_ID] = 0;

      // The return URL defaults to the <front> page.
      $nexturl = url("<front>", array("absolute" => TRUE));
      if (isset($_REQUEST['destination'])) {
        $nexturl = $_REQUEST['destination'];
      }

      $params = array("logout" => true, "NEXTURL" => $nexturl);
      $data = ConvioConfiguration::getInstance();
      $url = $data->getURI() . ConvioConstants::$LOGIN_SERVLET . "?" . http_build_query($params, NULL, "&");

      drupal_goto($url);
      return;
    }
  }

  /**
   * Converts the default user login form to the Convio login form.
   */
  public function getLoginForm(&$form, $form_state, $form_id) {

    if ($form_id != 'user_login_block' && $form_id != 'user_login') {
      return;
    }

    // Remove standard login form fields and links.
    unset($form["links"]);
    unset($form["name"]);
    unset($form["pass"]);
    unset($form["submit"]);

    $data = ConvioConfiguration::getInstance();
  
    // Form properties and listeners.
    $form["#action"] = $data->getURI() . ConvioConstants::$CR_CONS_API;
    $form["#id"] = "user-login-form";
    $form["#method"] = "POST";
    $form["#validate"] = array();
    $form["#submit"] = array();

    // Hidden fields.
    $form["v"] = array("#type" => "hidden", "#value" => ConvioConstants::$CONS_API_VERSION);
    $form["api_key"] = array("#type" => "hidden", "#value" => $data->getPublicKey());
    $form["method"] = array("#type" => "hidden", "#value" => "login");
    $form["sign_redirects"] = array("#type" => "hidden", "#value" => "true");

    // Success callback is the current page with the encrypted authentication credentials.
    $form["success_redirect"] = array("#type" => "hidden",
                                      "#value" => url("convio/session/validate",
                                                      array("absolute" => TRUE,
                                                            "query" => drupal_get_destination() . "&cons_id=\${loginResponse/cons_id}&token=\${loginResponse/token}"))
                                      );

    // Error callback is the user login page w/ errors.
    $form["error_redirect"] = array(
                                    "#type" => "hidden",
                                    "#value" => url("user/login",
                                                    array("absolute" => TRUE,
                                                          "query" => drupal_get_destination() . "&code=\${errorResponse/code}&message=\${errorResponse/message}"))
                                    );

    $form['user_name'] = array('#type' => 'textfield',
                               '#title' => t('Username'),
                               '#maxlength' => 60,
                               '#size' => ($form_id == 'user_login_block') ? 15 : 30,
                               '#required' => TRUE,
                               );

    $form['password'] = array('#type' => 'password',
                              '#title' => t('Password'),
                              '#maxlength' => 60,
                              '#size' => ($form_id == 'user_login_block') ? 15 : 30,
                              '#required' => TRUE,
                              );

    $form['submit'] = array('#type' => 'submit',
                            '#value' => t('Log in'),
                            );

    return $form;
  }

  /**
   * Validates the signature.
   */
  public function checkSignature() {
    $api_config = ConvioConfiguration::getInstance();

    // Generate an md5 hash of the query string minus the signature plus the secret key.
    // Note: Do not rely on $_SERVER["QUERY_STRING"] for Drupal.
    $sig = $_GET["signature"];
    $uri = $_SERVER["REQUEST_URI"];
    $pos = strpos($uri,"?");
    $query = substr($uri, $pos+1, strlen($uri));

    $pos = strpos($query,"&signature");
    $checkMe = substr($query, 0, $pos) . $api_config->getSecretKey();
    $md5 = md5($checkMe);
    if ($md5 != $sig) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Establishes a Drupal user session based on the COM user session.
   */
  public function newUserSession() {

    // Only establish the session if none is in place yet OR if there is a source code in the request URI.
    if (! isset($_SESSION[ConvioConstants::$SESSION_CONS_ID]) || $this->hasSourceCode()) {

      // Default the Drupal user to the anonymous user. 
      // If C360 is unresponsive, then the user will be treated as the anonymous user.
      $_SESSION[ConvioConstants::$SESSION_CONS_ID] = 0;

      // Synch the COM session with the Drupal session.
      // 1) Make loginTest API call to get the COM user.
      // 2) Redirect to //convio/session/validate to establish the Drupal user session.
      // 3) Redirect back to the original request URI.

      $api_config = ConvioConfiguration::getInstance();

      $auth_url = url("convio/session/validate", 
                      array("absolute" => TRUE,
                            "query" => $this->getDestination() . "&cons_id=\${loginResponse/cons_id}&token=\${loginResponse/token}"));

      $anon_url = url("convio/session/validate",
                      array("absolute" => TRUE,
                            "query" => $this->getDestination() . "&code=\${errorResponse/code}&message=\${errorResponse/message}"));

      $params = array("api_key" => $api_config->getPublicKey(),
                      "v" => ConvioConstants::$CONS_API_VERSION,
                      "method" => "loginTest",
                      "sign_redirects" => "true",
                      "success_redirect" => $auth_url,
                      "error_redirect" => $anon_url,
                      );
      $this->addSourceCodes($params);

      $url = $api_config->getURI() . ConvioConstants::$CR_CONS_API . "?" . http_build_query($params, NULL, "&");

      drupal_goto($url);
      return TRUE;
    }

    return FALSE;
  }

  private function hasSourceCode() {
    $has_source_codes = FALSE;
    $s_src = trim($_REQUEST["s_src"]);
    $s_subsrc = trim($_REQUEST["s_subsrc"]);
    if (! empty($s_src) || ! empty($s_subsrc)) {
      $has_source_codes = TRUE;
    }
    return $has_source_codes;
  }

  private function addSourceCodes(&$params) {
    $s_src = trim($_REQUEST["s_src"]);
    if (! empty($s_src)) {
      $params["s_src"] = $s_src;
    }

    $s_subsrc = trim($_REQUEST["s_subsrc"]);
    if (! empty($s_subsrc)) {
      $params["s_subsrc"] = $s_subsrc;
    }
  }

  /**
   * Like drupal_get_destination. Additionally strips out source code parameters.
   */
  private function getDestination() {
    if (isset($_REQUEST['destination'])) {
      $dest = $_REQUEST['destination'];
    } else {
      // Use $_GET here to retrieve the original path in source form.
      $dest = isset($_GET['q']) ? $_GET['q'] : '';
      $query = drupal_query_string_encode($_GET, array('q'));
      if ($query != '') {
        $dest .= '?'. $query;
      }
    }

    // Strip source code parameters from the URL.
    $dest = ConvioUtils::remove_querystring_var($dest, "s_src");
    $dest = ConvioUtils::remove_querystring_var($dest, "s_subsrc");

    return 'destination='. urlencode($dest);
  }

  /**
   * Establishes a Drupal user session.
   *
   * @param cons_id The cons ID
   * @return TRUE if the user was authenticated, otherwise FALSE.
   */
  public function authenticate($cons_id) {
    if ($cons_id == ConvioConstants::$SUPER_USER_ID) {
      // Prevent login as the Convio super user.
      watchdog("convio", "Authenticating as Convio super user is not supported.", array(), WATCHDOG_WARNING);
      return FALSE;

    } else {
      // Constituent.

      $convio_user = ConvioUser::find($cons_id);
      if ($convio_user == NULL) {
        watchdog("convio", "Failed to authenticate user: cons_id=" . $cons_id);
        return FALSE;
      }

      // Persist the user.
      $convio_user->save();

      // Do standard Drupal authentication.
      $username = $convio_user->getProperty("user_name");
      user_external_login_register($username, "convio");
      $edit = array();
      user_authenticate_finalize($edit);
      
      // Update Drupal session with Convio user session info.
      $_SESSION[ConvioConstants::$SESSION_CONS_ID] = $cons_id;
    }

    return TRUE;
  }

}
