<?php
/**
 * Copyright (C) 2010 Convio, Inc. All rights reserved.
 *
 * This program is free open source software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or (at your 
 * option) any later version. The GNU General Public License is available at 
 * http://www.gnu.org/licenses/. 
 *
 * =====================================================
 *
 * @file Single-sign-on (SSO), Constituent personalization implementation.
 * @version $Revision$
 */

class ConvioUser {

  private $cons_id;
  private $properties = NULL; // lazy-loaded
  private $groups     = NULL; // lazy-loaded
  private $interests  = NULL; // lazy-loaded


  /**
   * Resolves a cons_id to a ConvioUser, possibly null.
   *
   * @param cons_id The constituent ID
   * @return The user or null
   */
  public static function find($cons_id) {
    if ($cons_id == ConvioConstants::$SUPER_USER_ID) {
      // Don't bother making a getUser request. It won't work.
      return NULL;
    }

    $convio_user = NULL;
    try {
      $convio_user = new ConvioUser($cons_id);
    } catch (Exception $e) {
      // Do nothing. Return NULL.
    }

    return $convio_user;
  }

  function ConvioUser($cons_id) {
    $this->cons_id = $cons_id;
    $this->reload();
  }

  /**
   * Refreshes all user attributes.
   */
  public function reload() {
    // Reset properties.
    $this->properties = NULL;

    // Reset groups and interests.
    $this->groups = NULL;
    $this->interests = NULL;
  }

  /**
   * @return Fetches the cons_id.
   */
  public function getID() {
    return $this->cons_id;
  }

  /**
   * @return An array of key-value pairs representing user properties.
   */
  public function getProperties() {
    if ($this->properties == NULL) {
      $this->properties = array();

      $params = array("cons_id" => $this->cons_id);
      $settings = array("response_code" => "getConsResponse", 
                        "target" => ConvioConstants::$SR_CONS_API);

      $xml = ConvioAPIClient::post("getUser", $params, $settings);

      if ($xml !== FALSE) {
        foreach ($xml->children() as $childEl) {
          $this->loadProperties($childEl);
        }
      } else {
        $this->properties = NULL;
        throw new Exception("Failed to resolve user: cons_id=" . $this->cons_id);
      }

      $actual_cons_id = $this->properties["cons_id"];
      if ($this->cons_id != $actual_cons_id) {
        $this->properties = NULL;
        throw new Exception("cons_id mismatch: expected=" . $this->cons_id . "; actual=" . $actual_cons_id);
      }
    }
    return $this->properties;
  }

  private function loadProperties($el, $prefix = NULL) {
    $name = $el->getName();
    $children = $el->children();
    
    if (empty($children)) {
      // Leaf node.

      $key = ($prefix != NULL ? $prefix . "." : "") . $name;
      $value = $el;
      $this->properties[$key] = (string) $value;

    } else {
      // Not a leaf. This node is a complex attribute (e.g., address.street).

      foreach ($children as $childEl) {
        $this->loadProperties($childEl, $name);
      }
    }
  }

  /**
   * @param $name The property name
   * @return The property value
   */
  public function getProperty($name) {
    $this->getProperties();
    return $this->properties[$name];
  }

  /**
   * Indicates whether the user is a member of the C360 group.
   *
   * @param group_id The group ID
   * @return TRUE if the user is a member, otherwise FALSE.
   */
  public function isGroupMember($group_id) {
    foreach ($this->getGroups() as $id => $name) {
      if ($id == $group_id) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Indicates whether the user has the C360 interest.
   *
   * @param interest_id The interest ID
   * @return TRUE if the user has the interest, otherwise FALSE.
   */
  public function hasInterest($interest_id) {
    foreach ($this->getInterests() as $id => $name) {
      if ($id == $interest_id) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Lazy-loads user groups.
   */
  public function getGroups() {
    if ($this->groups == NULL) {

      $param = array("cons_id" => $this->cons_id);
      $settings = array("target" => ConvioConstants::$SR_CONS_API,
                        "response_code" => "getConsGroupsResponse");

      $xml = ConvioAPIClient::post("getUserGroups", $params, $settings);
      if ($xml !== FALSE) {

        $groups = array();
        foreach ($xml->group as $g) {
          $groups[(integer) $g->id] = t((string) $g->label);
        }
        $this->groups = $groups;
        watchdog("debug", print_r($groups, TRUE));

      } else {
        throw new Exception("Failed to retrieve C360 constituent groups. cons_id=" . $this->cons_id);
      }
    }

    return $this->groups;
  }

  /**
   * Lazy-loads user interests.
   */
  public function getInterests() {
    if ($this->interests == NULL) {

      $param = array("cons_id" => $this->cons_id);

      $settings = array("target" => ConvioConstants::$SR_CONS_API,
                        "response_code" => "getConsInterestsResponse");

      $xml = ConvioAPIClient::post("getUserInterests", $params, $settings);
      if ($xml !== FALSE) {

        $interests = array();
        foreach ($xml->interest as $i) {
          $interests[(integer) $i->id] = t((string) $idg->label);
        }
        $this->interests = $interests;
        watchdog("debug", print_r($interests, TRUE));

      } else {
        throw new Exception("Failed to retrieve C360 constituent interests. cons_id=" . $this->cons_id);
      }
    }

    return $this->interests;
  }

  /**
   * Persists the C360-Drupal user mapping.
   */
  public function save() {
    $cons_id = $this->cons_id;
    $username = $this->getProperty("user_name");
    $email = $this->getProperty("email.primary_address");
    $account = FALSE;

    watchdog("debug", print_r($this->getProperties(), TRUE));


    $result = db_query("SELECT user_id, cons_id FROM {convio_users} where cons_id = %d", $cons_id);
    $convio_user = db_fetch_object($result);
    if ($convio_user) {
      $account = user_load(array("uid" => $convio_user->user_id));
    } else {
      // Try and find the account via username.
      $account = user_load(array("name" => $username));
    }

    if (is_object($account) && $account->uid) {
      // Existing user.

      if ($cons_id == ConvioConstants::$SUPER_USER_ID) {
        // Prevent proxying the Convio super user.
        watchdog("convio", "Authenticating as Convio super user is not supported.", array(), WATCHDOG_SEVERE);
        return NULL;
      }

      $values = array(
                      "name" => $username,
                      "mail" => $email,
                      "convio_cons_id" => $cons_id,
                      "status" => 1, // the user should be unblocked
                      );

    } else {
      // New user.

      $values = array(
                      "name" => $username,
                      "pass" => user_password(), // generate a random password.
                      "mail" => $email,
                      "convio_cons_id" => $cons_id,
                      "init" => $email,
                      "status" => 1, // the user should be unblocked
                      );
    }

    // The convio implementation of hook_user should add convio_cons_id to the user account.
    return user_save($account, $values); 
  }

}
